@@ -28,20 +28,11 @@ the ``__view__`` attribute and insert your custom API views there.
 
 from flask import Blueprint, jsonify, request
 from os.path import abspath, dirname, join
-from sqlalchemy import Column, func
-from sqlalchemy.ext.declarative import declarative_base, declared_attr 
-from sqlalchemy.orm import class_mapper, mapperlib, Query
-from sqlalchemy.orm.attributes import InstrumentedAttribute
-from sqlalchemy.orm.collections import InstrumentedList
-from sqlalchemy.orm.dynamic import AppenderQuery
-from sqlalchemy.orm.mapper import Mapper
-from sqlalchemy.orm.properties import ColumnProperty, RelationshipProperty
+from sqlalchemy.orm import mapperlib
 from time import time
 from werkzeug.exceptions import HTTPException
 
-from ..project import current_project as pj
-from ..util import (Cacheable, _jsonify, JSONDepthExceededError,
-  JSONEncodedDict, Loggable, uncamelcase)
+from ..util import uncamelcase, query_to_models
 
 
 class APIError(HTTPException):
@@ -96,6 +87,8 @@ class API(object):
 
   """
 
+  __project__ = None
+
   _authorize = None
   _validate = None
 
@@ -191,32 +184,28 @@ class API(object):
         self.add_model(model_class, **kwargs)
 
   def _create_model_views(self, Model, options):
-    view = Model.__view__
     relationships = options.pop('relationships')
-    if 'collection' in view: view['collection'].attach_view(Model, **options)
-    if 'model' in view: view['model'].attach_view(Model, **options)
+    CollectionView.attach_view(Model, **options)
+    ModelView.attach_view(Model, **options)
     if relationships == True:
-      rels = Model.get_relationships()
+      rels = Model._relationships()
     elif relationships == False:
       rels = []
     else:
       rels = filter(
         lambda r: r.key in relationships,
-        Model.get_relationships()
+        Model._relationships()
       )
     for rel in rels:
       if rel.lazy == 'dynamic' and rel.uselist:
-        if 'relationship_model' in view:
-          view['relationship_model'].attach_view(rel, **options)
-        if 'dynamic_relationship' in view:
-          view['dynamic_relationship'].attach_view(rel, **options)
+        RelationshipModelView.attach_view(rel, **options)
+        DynamicRelationshipView.attach_view(rel, **options)
       elif rel.lazy in [True, 'select'] and rel.uselist:
-        if 'relationship_model' in view:
-          view['relationship_model'].attach_view(rel, **options)
-        if 'lazy_relationship' in view:
-          view['lazy_relationship'].attach_view(rel, **options)
+        RelationshipModelView.attach_view(rel, **options)
+        LazyRelationshipView.attach_view(rel, **options)
 
-  def _before_register(self, project):
+  def on_register(self, project):
+    self.__project__ = project
     self.blueprint = Blueprint(
       'api',
       project.config['PROJECT']['APP_FOLDER'] + '.api',
@@ -227,9 +216,9 @@ class API(object):
       self._create_model_views(model_class, options)
     IndexView.attach_view()
 
-  def _after_register(self, project):
-    Model.metadata.create_all(project._engine, checkfirst=True)
-    Model.query = _QueryProperty(project)
+    @project.before_startup
+    def handler(project):
+      project.app.register_blueprint(self.blueprint)
 
 # Views
 
@@ -356,11 +345,9 @@ class CollectionView(APIView):
 
   def get(self, parser, **kwargs):
     timers = {}
-    filtered_query = parser.filter_and_sort(self.Model.query)
+    filtered_query = parser.filter_and_sort(self.Model.q )
     now = time()
-    count = parser.filter_and_sort(
-      self.Model.query.get_count_query(), False
-    ).one()[0]
+    count = parser.filter_and_sort(self.Model.c, False).scalar()
     timers['count'] = time() - now
     now = time()
     content = [
@@ -406,17 +393,17 @@ class ModelView(APIView):
 
   def get_rule(self):
     url = '/%s' % self.Model.__tablename__
-    url += ''.join('/<%s>' % n for n in self.Model.get_primary_key_names())
+    url += ''.join('/<%s>' % k.name for k in self.Model._primary_keys())
     return url
 
   def get(self, parser, **kwargs):
-    model = self.Model.query.get(kwargs.values())
+    model = self.Model.q.get(kwargs.values())
     if not model:
       raise APIError(404, 'No resource found')
     return jsonify(model.jsonify(**parser.get_jsonify_kwargs()))
 
   def put(self, parser, **kwargs):
-    model = self.Model.query.get(kwargs.values())
+    model = self.Model.q.get(kwargs.values())
     if model:
       if self._is_valid(model.__class__, request.json):
         for k, v in request.json.items():
@@ -431,7 +418,7 @@ class ModelView(APIView):
     pass
 
   def delete(self, parser, **kwargs):
-    model = self.Model.query.get(kwargs.values())
+    model = self.Model.q.get(kwargs.values())
     if model:
       pj.session.delete(model)
       return jsonify({'status': '200 Success', 'content': 'Resource deleted'})
@@ -462,13 +449,13 @@ class RelationshipView(APIView):
   def get_rule(self):
     url = '/%s' % self.parent_Model.__tablename__
     url += ''.join(
-      '/<%s>' % n for n in self.parent_Model.get_primary_key_names()
+      '/<%s>' % k.name for k in self.parent_Model._primary_keys()
     )
     url += '/%s' % self.rel.key
     return url
 
   def get_collection(self, **kwargs):
-    parent_model = self.parent_Model.query.get(kwargs.values())
+    parent_model = self.parent_Model.q.get(kwargs.values())
     if not parent_model:
       raise APIError(404, 'No resource found')
     return getattr(parent_model, self.rel.key)
@@ -552,7 +539,7 @@ class RelationshipModelView(RelationshipView):
   def get_rule(self):
     url = '/%s' % self.parent_Model.__tablename__
     url += ''.join(
-      '/<%s>' % n for n in self.parent_Model.get_primary_key_names()
+      '/<%s>' % k.name for k in self.parent_Model._primary_keys()
     )
     url += '/%s/<key>' % self.rel.key
     return url
@@ -602,215 +589,6 @@ class IndexView(APIView):
       ]
     })
 
-# SQLAlchemy Model
-
-def get_model_classes_from_query(query):
-  return [
-    d['expr'].class_
-    for d in query.column_descriptions
-    if isinstance(d['expr'], Mapper)
-  ]
-
-class _BaseQuery(Query):
-
-  """Base query class.
-
-  The first two methods are copied from Flask-SQLAlchemy.
-
-  """
-
-  def get_or_404(self, model_id):
-    """Like get but aborts with 404 if not found."""
-    rv = self.get(model_id)
-    if rv is None:
-      abort(404)
-    return rv
-
-  def first_or_404(self):
-    """Like first but aborts with 404 if not found."""
-    rv = self.first()
-    if rv is None:
-      abort(404)
-    return rv
-
-  def get_count_query(self):
-    """Get correspondint count query.
-
-    This should be used for fast counts. Apply filters to it and apply the
-    method ``one()`` to bypass the use of subqueries (which are highly
-    inefficient in MySQL).
-    
-    """
-    models = self.get_models()
-
-    # _BaseQuery objects should only ever have one model
-    assert len(models) == 1, '%s models found for %s' % (len(models), self)
-
-    Model = models[0]
-    query = pj.session.query(func.count(Model)).select_from(Model)
-    return query
-
-  def get_models(self):
-    return get_model_classes_from_query(self)
-
-class _QueryProperty(object):
-
-  def __init__(self, project):
-    self.project = project
-
-  def __get__(self, obj, cls):
-    try:
-      mapper = class_mapper(cls)
-      if mapper:
-        return _BaseQuery(mapper, session=self.project.session())
-    except UnmappedClassError:
-      return None
-
-class ExpandedBase(Cacheable, Loggable):
-
-  """Adding a few features to the declarative base.
-
-  Currently:
-
-  * Automatic table naming
-  * Caching
-  * Jsonifying
-  * Logging
-
-  """
-
-  __view__ = {
-    'model': ModelView,
-    'collection': CollectionView,
-    'dynamic_relationship': DynamicRelationshipView,
-    'lazy_relationship': LazyRelationshipView,
-    'relationship_model': RelationshipModelView
-  }
-
-  _cache = Column(JSONEncodedDict)
-  _json_depth = 0
-
-  query = None
-
-  def __init__(self, **kwargs):
-    for k, v in kwargs.items():
-      setattr(self, k, v)
-
-  def __repr__(self):
-    primary_keys = ', '.join(
-      '%s=%r' % (k, getattr(self, k))
-      for k in self.__class__.get_primary_key_names()
-    )
-    return '<%s (%s)>' % (self.__class__.__name__, primary_keys)
-
-  @declared_attr
-  def __json__(cls):
-    """Varnames that get JSONified.
-
-    Doesn't emit any additional queries!
-
-    """
-    return list(
-      varname
-      for varname in dir(cls)
-      if not varname.startswith('_')  # don't show private properties
-      if (
-        isinstance(getattr(cls, varname), property) 
-      ) or (
-        isinstance(getattr(cls, varname), InstrumentedAttribute) and
-        isinstance(getattr(cls, varname).property, ColumnProperty)
-      ) or (
-        isinstance(getattr(cls, varname), InstrumentedAttribute) and
-        isinstance(getattr(cls, varname).property, RelationshipProperty)
-        and (
-          getattr(cls, varname).property.lazy in [False, 'joined', 'immediate']
-        )
-      )
-    )
-
-  @declared_attr
-  def __tablename__(cls):
-    """Automatically create the table name.
-
-    Override this to choose your own tablename (e.g. for single table
-    inheritance).
-
-    """
-    return '%ss' % uncamelcase(cls.__name__)
-
-  def jsonify(self, depth=1, expand=True):
-    """Special implementation of jsonify for Model objects.
-    
-    Overrides the basic jsonify method to specialize it for models.
-
-    This function minimizes the number of lookups it does (no dynamic
-    type checking on the properties for example) to maximize speed.
-
-    :param depth:
-    :type depth: int
-    :rtype: dict
-
-    """
-    if depth <= self._json_depth:
-      # this instance has already been jsonified at a greater or
-      # equal depth, so we simply return its key (only used if expand is False)
-      return self.get_primary_keys()
-    if not expand:
-      self._json_depth = depth
-    rv = {}
-    for varname in self.__json__:
-      try:
-        rv[varname] = _jsonify(getattr(self, varname), depth - 1, expand)
-      except ValueError as e:
-        rv[varname] = e.message
-      except JSONDepthExceededError:
-        pass
-    return rv
-
-  def get_primary_keys(self):
-    return dict(
-      (k, getattr(self, k))
-      for k in self.__class__.get_primary_key_names()
-    )
-
-  @classmethod
-  def find_or_create(cls, **kwargs):
-    instance = self.filter_by(**kwargs).first()
-    if instance:
-      return instance, False
-    instance = cls(**kwargs)
-    session = cls.query.db.session
-    session.add(instance)
-    session.flush()
-    return instance, True
-
-  @classmethod
-  def get_columns(cls, show_private=False):
-    columns = class_mapper(cls).columns
-    if not show_private:
-      columns = [c for c in columns if not c.key.startswith('_')]
-    return columns
-
-  @classmethod
-  def get_primary_key_names(cls):
-    return [key.name for key in class_mapper(cls).primary_key]
-
-  @classmethod
-  def get_relationships(cls, show_private=False):
-    rels =  class_mapper(cls).relationships.values()
-    if not show_private:
-      rels = [rel for rel in rels if not rel.key.startswith('_')]
-    return rels
-
-  @classmethod
-  def get_related_models(cls, show_private=False):
-    return [
-      (r.key, r.mapper.class_)
-      for r in cls.get_relationships(show_private)
-    ]
-
-Model = declarative_base(cls=ExpandedBase)
-
 # Helper
 
 class Parser(object):
@@ -895,7 +673,7 @@ class Parser(object):
 
   def _get_model_class(self, query):
   
-    models = get_model_classes_from_query(query)
+    models = query_to_models(query)
 
     # only tested for _BaseQueries and associated count queries
     assert len(models) < 2, 'Invalid query'
@@ -903,8 +681,6 @@ class Parser(object):
     if not len(models):
       # this is a count query
       return query._select_from_entity
-    # this is a _BaseQuery
+    # this is a Query
     return models[0]
 
-
-
