@@ -2,23 +2,34 @@
 
 """Project module."""
 
-from celery.signals import task_postrun
 from ConfigParser import SafeConfigParser
-from flask import abort
 from os.path import abspath, dirname, join, sep, split, splitext
 from re import match, sub
 from sqlalchemy import create_engine  
 from sqlalchemy.exc import InvalidRequestError
-from sqlalchemy.orm import scoped_session, sessionmaker
-from weakref import proxy
+from sqlalchemy.orm import Query, scoped_session, sessionmaker
+from sys import path
+from threading import local
 from werkzeug.local import LocalProxy
 
 from .util import convert
 
+
+class _Tls(local):
+
+  """Thread local storage for the project state."""
+  
+  _state = {}
+  _current_project = None
+
+_tls = _Tls()
+
+
 class ProjectImportError(Exception):
 
   pass
 
+
 class Project(object):
 
   """Project class.
@@ -28,8 +39,6 @@ class Project(object):
   
   """
 
-  __current__ = None
-
   config = {
     'PROJECT': {
       'NAME': '',
@@ -44,7 +53,7 @@ class Project(object):
     'ENGINE': {
       'URL': 'sqlite://',
     },
-    'APP': {
+    'FLASK': {
       'SECRET_KEY': 'a_default_unsafe_key',
     },
     'CELERY': {
@@ -54,34 +63,49 @@ class Project(object):
     },
   }
 
-  def __init__(self, config_path):
+  def __init__(self, config_path=None, make=True):
 
-    config = self._parse_config(config_path)
-    for key in config:
-      if key in self.config:
-        self.config[key].update(config[key])
-      else:
-        self.config[key] = config[key]
+    self.__dict__ = _tls._state
 
-    self.root_dir = dirname(abspath(config_path))
-    self.domain = (
-      self.config['PROJECT']['DOMAIN'] or
-      sub(r'\W+', '_', self.config['PROJECT']['NAME'].lower())
-    )
-    self.subdomain = (
-      self.config['PROJECT']['SUBDOMAIN'] or
-      splitext(config_path)[0].replace(sep, '-')
-    )
+    if _tls._current_project is None:
 
-    assert Project.__current__ is None, 'More than one project.'
-    Project.__current__ = proxy(self)
+      if config_path is None:
+        raise ProjectImportError('Project instantiation outside the Flasker '
+                                 'command line tool requires a configuration '
+                                 'file path.')
 
-    self.app = None
-    self.celery = None
-    self.session = None
-    self._engine = None
-    self._extensions = []
-    self._before_startup = []
+      config = self._parse_config(config_path)
+      for key in config:
+        if key in self.config:
+          self.config[key].update(config[key])
+        else:
+          self.config[key] = config[key]
+
+      self.root_dir = dirname(abspath(config_path))
+      self.domain = (
+        self.config['PROJECT']['DOMAIN'] or
+        sub(r'\W+', '_', self.config['PROJECT']['NAME'].lower())
+      )
+      self.subdomain = (
+        self.config['PROJECT']['SUBDOMAIN'] or
+        splitext(config_path)[0].replace(sep, '-')
+      )
+
+      path.append(self.root_dir)
+
+      self.app = None
+      self.cel = None
+      self.session = None
+
+      self._engine = None
+      self._query_class = Query
+      self._extensions = []
+      self._before_startup = []
+
+      _tls._current_project = self
+
+      if make:
+        self._make()
 
   def __repr__(self):
     return '<Project %r, %r>' % (self.config['PROJECT']['NAME'], self.root_dir)
@@ -94,47 +118,49 @@ class Project(object):
     """Decorator, hook to run a function right before project starts."""
     self._before_startup.append(func)
 
-  def _make(self, app=False, celery=False):
+  def _make(self):
     """Create all project components."""
+
     # core
-    for mod in  ['app', 'celery']:
+    for mod in  ['flask', 'celery']:
       __import__('flasker.core.%s' % mod)
+
     # project modules
     project_modules = self.config['PROJECT']['MODULES'].split(',') or []
     for mod in project_modules:
       __import__(mod.strip())
-    # database
-    self._setup_database_connection()
+
     # extensions
     for extension, config_section in self._extensions or []:
       if config_section:
         for k, v in self.config[config_section].items():
           extension.config[k] = v
-      extension._before_register(self)
-      self.app.register_blueprint(extension.blueprint)
-      extension._after_register(self)
+      extension.on_register(self)
+
+    # database
+    self._setup_database_connection()
+
     # final hook
     for func in self._before_startup or []:
       func(self)
 
   def _setup_database_connection(self):
+    """Setup the database engine."""
     engine_ops = dict((k.lower(), v) for k,v in self.config['ENGINE'].items())
     self._engine = create_engine(engine_ops.pop('url'), **engine_ops)
-    self.session = scoped_session(sessionmaker(bind=self._engine))
+    self.session = scoped_session(
+      sessionmaker(bind=self._engine, query_cls=self._query_class)
+    )
 
   def _dismantle_database_connections(self):
-    """Remove database connections.
-
-    Has to be called after app request/job terminates or connections
-    will leak.
-
-    """
+    """Remove database connections."""
     try:
       if self.config['PROJECT']['COMMIT_ON_TEARDOWN']:
         self.session.commit()
     except InvalidRequestError as e:
       self.session.rollback()
       self.session.expunge_all()
+      raise e
     finally:
       self.session.remove()
 
@@ -158,15 +184,13 @@ class Project(object):
       (s, dict((k, convert(v)) for (k, v) in parser.items(s)))
       for s in parser.sections()
     )
-    # some conf checking
     if not conf['PROJECT']['NAME']:
       raise ProjectImportError('Missing project name.')
     return conf
 
-  @classmethod
-  def _get_current_project(cls):
-    """Hook for ``current_project`` proxy."""
-    return Project.__current__
 
-current_project = LocalProxy(lambda: Project._get_current_project())
+def _get_current_project():
+  return _tls._current_project
+
+current_project = LocalProxy(_get_current_project)
 
