@@ -12,6 +12,7 @@ from math import ceil
 from re import sub
 from sqlalchemy.ext.mutable import Mutable
 from sqlalchemy.orm import Query
+from sqlalchemy.orm.mapper import Mapper
 from sqlalchemy.types import TypeDecorator, UnicodeText
 from time import time
 
@@ -22,34 +23,6 @@ from time import time
 #
 # ===
 
-def prod(iterable, key=None):
-  """Cumulative product function (the equivalent of ``sum``).
-
-  :param key: function called on each element of the iterable, if none then
-    identity is assumed
-  :type key: callable
-  :rtype: int, float
-
-  """
-  rv = 1
-  for elem in iterable:
-    if key is None:
-      rv *= elem
-    else:
-      rv *= key(elem)
-  return rv
-
-def uncamelcase(name):
-  """Transforms CamelCase to underscore_case.
-
-  :param name: string input
-  :type name: str
-  :rtype: str
-  
-  """
-  s1 = sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
-  return sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
-
 def convert(value, rtype=None):
   """Converts a string to another value.
 
@@ -158,6 +131,34 @@ def partition(collection, parts=0, size=0):
       yield islice(collection, offset, offset+limit), Part(offset, limit)
       offset += limit
 
+def prod(iterable, key=None):
+  """Cumulative product function (the equivalent of ``sum``).
+
+  :param key: function called on each element of the iterable, if none then
+    identity is assumed
+  :type key: callable
+  :rtype: int, float
+
+  """
+  rv = 1
+  for elem in iterable:
+    if key is None:
+      rv *= elem
+    else:
+      rv *= key(elem)
+  return rv
+
+def uncamelcase(name):
+  """Transforms CamelCase to underscore_case.
+
+  :param name: string input
+  :type name: str
+  :rtype: str
+  
+  """
+  s1 = sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
+  return sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
+
 class Dict(dict):
 
   """Expands the dictionary class with a few helper methods.
@@ -309,6 +310,7 @@ class Dict(dict):
         items.append(row)
     return items
 
+
 class SmartDictReader(DictReader):
 
   """``DictReader`` with built-in value conversion.
@@ -370,6 +372,7 @@ class SmartDictReader(DictReader):
       self.rows_imported += 1
       return processed_row
 
+
 # ===
 # 
 #   Mixins
@@ -462,6 +465,7 @@ class Cacheable(object):
     """
     return _CachedProperty(func)
 
+
 class _CacheRefresh(object):
 
   """Special class used to trigger cache refreshes."""
@@ -469,6 +473,7 @@ class _CacheRefresh(object):
   def __init__(self, expiration):
     self.expiration = expiration
 
+
 class _CachedProperty(property):
 
   """Instance of a cached property for a model.
@@ -514,12 +519,14 @@ class _CachedProperty(property):
   def __repr__(self):
     return '<CachedProperty %r>' % self.func
 
+
 # Jsonifying
 
 class JSONDepthExceededError(Exception):
 
   pass
 
+
 def _jsonify(value, depth, expand):
   if depth < 0:
     raise JSONDepthExceededError()
@@ -537,6 +544,7 @@ def _jsonify(value, depth, expand):
     return None
   raise ValueError('not jsonifiable')
 
+
 class Jsonifiable(object):
 
   """JSONification mixin."""
@@ -578,6 +586,7 @@ class Jsonifiable(object):
         pass
     return rv
 
+
 # Logging
 
 class Loggable(object):
@@ -619,6 +628,7 @@ class Loggable(object):
     """Error level message."""
     return self._logger(message, 'error')
 
+
 # ===
 #
 #   SQLAlchemy custom
@@ -648,6 +658,7 @@ class _JSONEncodedType(TypeDecorator):
   def process_result_value(self, value, dialect):
     raise NotImplementedError()
 
+
 class JSONEncodedDict(_JSONEncodedType):
 
   """Implements dictionary column field type for SQLAlchemy.
@@ -668,6 +679,7 @@ class JSONEncodedDict(_JSONEncodedType):
   def process_result_value(self, value, dialect):
     return loads(value) if value else {}
 
+
 class _MutableDict(Mutable, dict):
 
   """Used with JSONEncoded dict to be able to track updates.
@@ -705,6 +717,7 @@ class _MutableDict(Mutable, dict):
     
 _MutableDict.associate_with(JSONEncodedDict)
 
+
 class JSONEncodedList(_JSONEncodedType):
 
   """Implements list column field type for SQLAlchemy.
@@ -713,11 +726,15 @@ class JSONEncodedList(_JSONEncodedType):
 
     some_column_name = Column(JSONEncodedList)
 
+  Currently only set, delete, append and extend events are tracked. Others
+  will require a call to ``changed`` to be persisted.
+
   """
 
   def process_result_value(self, value, dialect):
     return loads(value) if value else []
 
+
 class _MutableList(Mutable, list):
 
   """Used with JSONEncoded list to be able to track updates.
@@ -738,42 +755,152 @@ class _MutableList(Mutable, list):
     else:
       return value
 
-  # TODO: actual mutability tracking
+  def append(self, *args, **kwargs):
+    """Detect update events and emit change events."""
+    list.append(self, *args, **kwargs)
+    self.changed()
+
+  def extend(self, *args, **kwargs):
+    """Detect update events and emit change events."""
+    list.extend(self, *args, **kwargs)
+    self.changed()
     
+  def __setitem__(self, index, value):
+    """Detect set events and emit change events."""
+    list.__setitem__(self, index, value)
+    self.changed()
+    
+  def __delitem__(self, index):
+    """Detect del events and emit change events."""
+    list.__delitem__(self, index)
+    self.changed()
+
 _MutableList.associate_with(JSONEncodedList)
 
+
+# Query helpers
+
+def query_to_models(query):
+  """Returns the model classes associated with a query.
+  
+  :param query: the query to be executed
+  :type query: sqlalchemy.orm.query.Query
+  :rtype: list
+
+  """
+  return [
+    d['expr'].class_
+    for d in query.column_descriptions
+    if isinstance(d['expr'], Mapper)
+  ]
+
+def query_to_dataframe(query, connection=None, exclude=None, index=None,
+                       columns=None, coerce_float=False):
+  """Load a Pandas dataframe from an SQLAlchemy query.
+
+  :param query: the query to be executed
+  :type query: sqlalchemy.orm.query.Query
+  :param connection: the connection to use to execute the query. By default
+    the method will use the query's session's current connection. Note that
+    connection is left open afterwards.
+  :type connection: sqlalchemy.engine.base.Connection
+  :param exclude: a list of column names to exclude from the dataframe
+  :type exclude: list
+  :param index: the column to use as index
+  :type index: str
+  :param names: a list of column names. If unspecified, the method will use
+    the table's keys from the query's metadata. If the passed data do not have
+    named associated with them, this argument provides names for the columns.
+    Otherwise this argument indicates the order of the columns in the result
+    (any names not found in the data will become all-NA columns)
+  :type names: list
+  :param coerce_float: Attempt to convert values to non-string, non-numeric
+    objects (like decimal.Decimal) to floating point.
+  :type coerce_float: bool
+  :rtype: pandas.DataFrame
+  
+  """
+  from pandas import DataFrame
+  connection = connection or query._connection_from_session()
+  exclude = exclude or []
+  result = connection.execute(query.statement)
+  columns = columns or result.keys()
+  dataframe = DataFrame.from_records(
+    result.fetchall(),
+    columns=columns,
+    exclude=exclude,
+    index=index,
+    coerce_float=coerce_float,
+  )
+  return dataframe
+
+
 # ===
 #
 # Computations
 #
 # ===
 
-class RunningStatistic(object):
+class RunningStat(object):
 
-  """ To compute running statistics efficiently."""
+  """ To compute running means and variances efficiently.
+
+  Usage::
+
+    rs = RunningStat()
+    for i in range(10):
+      rs.push(i)
+    rs.var
+
+  """
 
   def __init__(self):
     self.count = 0
-    self.mean = float(0)
+    self._mean = float(0)
     self.unweighted_variance = float(0)
 
+  def __repr__(self):
+    return '<RunningStat (count=%s, avg=%s, sdv=%s)>' % (
+      self.count, self.avg, self.sdv
+  )
+
+  @property
+  def avg(self):
+    """Current mean."""
+    if self.count > 0:
+      return self._mean
+    return 0
+
+  @property
+  def var(self):
+    """Current variance."""
+    if self.count > 1:
+      return self.unweighted_variance/(self.count-1)
+    return 0
+
+  @property
+  def sdv(self):
+    """Current standard deviation."""
+    return self.var ** 0.5
+
   def push(self, n):
+    """Add a new element to the statistic.
+
+    :param n: number to add
+    :type n: int, float
+
+    """
     if n == None:
       return
     self.count += 1
     if self.count == 1:
-      self.mean = float(n)
+      self._mean = float(n)
       self.unweighted_variance = float(0)
     else:
-      mean = self.mean
+      mean = self._mean
       s = self.unweighted_variance
-      self.mean = mean + (n - mean) / self.count
-      self.unweighted_variance = s + (n - self.mean) * (n - mean)
-
-  def variance(self):
-      if self.count>1:
-        return self.unweighted_variance/(self.count-1)
-      return 0
+      self._mean = mean + (n - mean) / self.count
+      self.unweighted_variance = s + (n - self._mean) * (n - mean)
 
 def exponential_smoothing(data, alpha=0.5):
   """Smoothing data.
@@ -877,27 +1004,3 @@ def histogram(data, key=None, bins=50, restrict=None, categories=None,
       )
     return data_histogram
 
-def query_to_dataframe(query, connection=None, exclude=None):
-  """Load a Pandas dataframe from an SQLAlchemy query.
-
-  :param query: the query to be executed
-  :type query: sqlalchemy.orm.query.Query
-  :param connection: the connection to use to execute the query. By default
-    the method will use the query's session's current connection. Note that
-    connection is left open afterwards.
-  :type connection: sqlalchemy.engine.base.Connection
-  :param exclude: a list of column names to exclude from the dataframe
-  :type exclude: list
-  :rtype: pandas.DataFrame
-  
-  """
-  from pandas import DataFrame
-  connection = connection or query._connection_from_session()
-  exclude = exclude or []
-  result = connection.execute(query.statement)
-  dataframe = DataFrame.from_records(
-    result.fetchall(),
-    columns=result.keys(),
-    exclude=exclude,
-  )
-  return dataframe
